!-----------------------------------------------------------------------
!
! convert sparse matrix into format suitable for matlab input.
!
!-----------------------------------------------------------------------
!
      include 'common.h'
!
      !---character variables----------
      character      title*72 , key*8    , mxtype*3 , rhstyp*3,
     &               ptrfmt*16, indfmt*16, valfmt*20, rhsfmt*20
      character      charspec*6
!
      !---integer variables----------
      integer        totcrd, ptrcrd, indcrd, valcrd, rhscrd,
     &               nrow  , ncol  , nnzero, neltvl,
     &               nrhs  , nrhsix, nrhsvl, nguess, nexact
      integer i, j
!
      !---real variables----------
      real(z) zero
!
      !---integer arrays----------
      integer, allocatable :: pointr(:), rowind(:)
      integer, allocatable :: rhsptr(:), rhsind(:)
!
      !---real arrays----------
      real(z), allocatable :: values(:), rhsval(:), xexact(:), sguess(:)
!
!-----------------------------------------------------------------------
!
      zero = 0.

!     ------------------------
!     ... read in header block
!     ------------------------

      read ( 5, 1000 ) title , key   ,
     &                     totcrd, ptrcrd, indcrd, valcrd, rhscrd,
     &                     mxtype, nrow  , ncol  , nnzero, neltvl,
     &                     ptrfmt, indfmt, valfmt, rhsfmt

      if ( rhscrd .gt. 0 )
     &  read ( 5, 1001 ) rhstyp, nrhs, nrhsix

 1000 format ( a72, a8 / 5i14 / a3, 11x, 4i14 / 2a16, 2a20 )
 1001 format ( a3, 11x, 2i14 )

!     -------------------------
!     ... allocations etc.
!     -------------------------

!
!     if ( nrow .ne. ncol ) then
!       write (*,*) 'Error: matrix not square.'
!       goto 999
!     endif
!
      i = index('ABCDEFGHIJKLMNOPQRSTUVWXYZ',mxtype(1:1))
      if (i .ne. 0) mxtype(1:1) = 'abcdefghijklmnopqrstuvwxyz'(i:i)
      i = index('ABCDEFGHIJKLMNOPQRSTUVWXYZ',mxtype(2:2))
      if (i .ne. 0) mxtype(2:2) = 'abcdefghijklmnopqrstuvwxyz'(i:i)
      i = index('ABCDEFGHIJKLMNOPQRSTUVWXYZ',mxtype(3:3))
      if (i .ne. 0) mxtype(3:3) = 'abcdefghijklmnopqrstuvwxyz'(i:i)
      i = index('ABCDEFGHIJKLMNOPQRSTUVWXYZ',rhstyp(1:1))
      if (i .ne. 0) rhstyp(1:1) = 'abcdefghijklmnopqrstuvwxyz'(i:i)
      i = index('ABCDEFGHIJKLMNOPQRSTUVWXYZ',rhstyp(2:2))
      if (i .ne. 0) rhstyp(2:2) = 'abcdefghijklmnopqrstuvwxyz'(i:i)
      i = index('ABCDEFGHIJKLMNOPQRSTUVWXYZ',rhstyp(3:3))
      if (i .ne. 0) rhstyp(3:3) = 'abcdefghijklmnopqrstuvwxyz'(i:i)
!
      if ( mxtype(1:1) .ne. 'R' .and. mxtype(1:1) .ne. 'r' ) then
        write (*,*) 'Error: matrix must have real entries.'
        goto 999
      endif
!
      if ( valcrd.le.0 .and. nnzero.ne.0) then
        write (*,*) 'Error: matrix entries not present.'
        goto 999
      endif
!
      if ( mxtype (3:3) .ne. 'a' ) then
        write (*,*) 'Error: cannot handle unassembled f.e. matrix.'
        goto 999
      endif
!
      if ( rhscrd .gt. 0 ) then
      if ( rhstyp (1:1) .ne. 'f' ) then
        write (*,*) 'Error: cannot handle sparse vectors.'
        goto 999
      endif
      endif
!
          allocate (pointr(ncol+1))
          allocate (rowind(nnzero))
          allocate (values(nnzero))
      if ( nrhs .gt. 0 )  then
          allocate (rhsval(nrow*nrhs))
        if ( rhstyp(2:2) .eq. 'g' ) then
          allocate (sguess(nrow*nrhs))
        endif
        if ( rhstyp(3:3) .eq. 'x' ) then
          allocate (xexact(nrow*nrhs))
        endif
      endif

!     -------------------------
!     ... read matrix structure
!     -------------------------

      read ( 5, ptrfmt ) ( pointr (i), i = 1, ncol+1 )

      read ( 5, indfmt ) ( rowind (i), i = 1, nnzero )

      if  ( valcrd .gt. 0 )  then

!         ----------------------
!         ... read matrix values
!         ----------------------

          if  ( mxtype (3:3) .eq. 'a' )  then
              read ( 5, valfmt ) ( values (i), i = 1, nnzero )
          else
              read ( 5, valfmt ) ( values (i), i = 1, neltvl )
          endif

!         -------------------------
!         ... read right-hand sides
!         -------------------------

          if  ( nrhs .gt. 0 )  then

              if  ( rhstyp(1:1) .eq. 'f' ) then

!                 -------------------------------
!                 ... read dense right-hand sides
!                 -------------------------------

                  nrhsvl = nrow * nrhs
                  read ( 5, rhsfmt ) ( rhsval (i), i = 1, nrhsvl )

              else

!                 ---------------------------------------------
!                 ... read sparse or elemental right-hand sides
!                 ---------------------------------------------


                  if (mxtype(3:3) .eq. 'a') then

!                    ------------------------------------------------
!                    ... sparse right-hand sides - read pointer array
!                    ------------------------------------------------

                     read (5, ptrfmt) ( rhsptr (i), i = 1, nrhs+1 )

!                    ----------------------------------------
!                    ... read sparsity pattern for right-hand
!                        sides
!                    ----------------------------------------

                     read (5, indfmt) ( rhsind (i), i = 1, nrhsix )

!                    --------------------------------------
!                    ... read sparse right-hand side values
!                    --------------------------------------

                     read (5, rhsfmt) ( rhsval (i), i = 1, nrhsix )

                  else

!                    -----------------------------------
!                    ... read elemental right-hand sides
!                    -----------------------------------

                     nrhsvl = nnzero * nrhs
                     read (5, rhsfmt) ( rhsval (i), i = 1, nrhsvl )

                  endif

              endif

              if  ( rhstyp(2:2) .eq. 'g' ) then

!                 -------------------------
!                 ... read starting guesses
!                 -------------------------

                 nguess = nrow * nrhs
                 read (5, rhsfmt) ( sguess (i), i = 1, nguess )

              endif

              if  ( rhstyp(3:3) .eq. 'x' ) then

!                 -------------------------
!                 ... read solution vectors
!                 -------------------------

                 nexact = nrow * nrhs
                 read (5, rhsfmt) ( xexact (i), i = 1, nexact )
              endif
          endif
      endif

!
!-----------------------------------------------------------------------
!
!
      !---produce output----------
!
      do i = 1, ncol
      do j = pointr(i), pointr(i+1)-1
        write (*,'(2i12,1x,e20.10)') rowind(j), i, values(j)
      enddo
      enddo
!
!-----------------------------------------------------------------------
!

!     -------------------------
!     ... deallocations etc.
!     -------------------------

          deallocate (pointr)
          deallocate (rowind)
          deallocate (values)
      if ( nrhs .gt. 0 )  then
          deallocate (rhsval)
        if ( rhstyp(2:2) .eq. 'g' ) then
          deallocate (sguess)
        endif
        if ( rhstyp(3:3) .eq. 'x' ) then
          deallocate (xexact)
        endif
      endif

!
!---all done----------
!
 999  continue
      end
!
!-----------------------------------------------------------------------
